using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace LD31
{
    // THEME: Entire game on one screen
    public class Main : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        KeyPressed spacebar = new KeyPressed();

        Agent[] agents;
        World world;

        bool gameover = false;

        // Window stuff
        public const ushort ScreenWidth = 768;
        public const ushort ScreenHeight = 768;

        public Main()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            IsFixedTimeStep = false;
        }

        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = ScreenWidth;
            graphics.PreferredBackBufferHeight = ScreenHeight;

            graphics.ApplyChanges();

            world = new World(); // that was easy

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Texture2D snowman = Content.Load<Texture2D>("tex/snowman");

            agents = new Agent[] {
                new Agent(Content.Load<Texture2D>("tex/player"), new Vector2(32, 32)),
                new Agent(snowman, new Vector2(ScreenWidth - 64, ScreenHeight - 64)),
                new Agent(snowman, new Vector2(32, ScreenHeight - 64)),
                new Agent(snowman, new Vector2(ScreenWidth - 64, 32))
            };

            world.LoadContent(Content);
            Sound.LoadContent(Content);
        }

        protected override void Update(GameTime gameTime)
        {
            float deltams = (float)gameTime.ElapsedGameTime.TotalMilliseconds;

            KeyboardState state = Keyboard.GetState();

            if (state.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            UpdateAgents(deltams, state);

            world.Update(deltams, ref agents, ref gameover);

            base.Update(gameTime);
        }

        void UpdateAgents(float delta, KeyboardState state)
        {
            if (!gameover)
            {
                Dimension2D dir = new Dimension2D();

                if (state.IsKeyDown(Keys.Up))
                {
                    dir.Y = -1;
                }
                else if (state.IsKeyDown(Keys.Down))
                {
                    dir.Y = 1;
                }

                if (state.IsKeyDown(Keys.Left))
                {
                    dir.X = -1;
                }
                else if (state.IsKeyDown(Keys.Right))
                {
                    dir.X = 1;
                }

                agents[0].Direction = dir;

                if (spacebar.Pressed(state.IsKeyDown(Keys.Space)))
                {
                    agents[0].PlaceBomb(ref world, 0);
                }
            }

            world.DoAi(ref agents, ref world);

            for (byte i = 0; i < agents.Length; i++)
            {
                if (agents[i] != null) agents[i].Update(delta, ref world);
            }
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);


            spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.NonPremultiplied, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);

            world.Draw(spriteBatch);

            for (byte i = 0; i < agents.Length; i++)
            {
                if (agents[i] != null)
                    agents[i].Draw(spriteBatch);
            }

            spriteBatch.End();


            base.Draw(gameTime);
        }
    }
}
