﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;

namespace LD31
{
    static class Sound
    {
        private sealed class SoundManager
        {
            private class Instance
            {
                private SoundEffect sound;
                public SoundEffect Sound { get { return sound; } }

                public Instance(SoundEffect s)
                {
                    this.sound = s;
                }
            }

            private static IDictionary<string, Instance> dict = new Dictionary<string, Instance>();

            public static SoundEffect Get(string name)
            {
                return dict[name].Sound;
            }

            public static void Load(ContentManager c)
            {
                dict["explosion"] = new Instance(c.Load<SoundEffect>("sound/explosion"));
                dict["powerup"] = new Instance(c.Load<SoundEffect>("sound/powerup"));
            }
        }

        private static readonly Random ran;

        static Sound()
        {
            ran = new Random();
        }

        public static void LoadContent(ContentManager c)
        {
            SoundManager.Load(c);
        }

        public static void PlayRandom(string name)
        {
            Play(name, (float)ran.Next(-20, 21) / 100.0f);
        }

        public static void Play(string name, float pitch = 0)
        {
            SoundEffectInstance i = SoundManager.Get(name).CreateInstance();
            i.Pitch = pitch;

            i.Play();
        }
    }
}
