﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace LD31
{
    // Let's handle items here too!
    class World
    {
        class Aiproc
        {
            public bool bombAction = false;
            public bool justBombed = false;
            public byte prefer = 0;

            public Dimension2D dir;
            public Dimension2D bombplaceTile;
            public Dimension2D startingTile;
            public Dimension2D coverTile;
        }

        public const byte SIZE = Main.ScreenWidth / BLOCK_SIZE;
        public const byte BLOCK_SIZE = 32;

        Tile[,] blocks;
        Texture2D[] textures;
        Aiproc[] procs;

        Random ran;
        TextManager textManager;

        List<FireParticle> fires = new List<FireParticle>();


        public void Reset(Dimension2D pos)
        {
            blocks[pos.X, pos.Y].Reset();
        }

        public void Set(Dimension2D pos, Tile b)
        {
            blocks[pos.X, pos.Y] = b;
        }

        public bool CanWalkHere(Dimension2D pos)
        {
            return blocks[pos.X, pos.Y].id == 0 || blocks[pos.X, pos.Y].id > 2;
        }
        public bool IsThereBlockAt(Dimension2D pos)
        {
            return blocks[pos.X, pos.Y].id != 0;
        }

        public Dimension2D GetBlockPos(Dimension2D pos, Dimension2D dir)
        {
            return new Dimension2D(pos.X + dir.X, pos.Y + dir.Y);
        }
        public Dimension2D[] GetSurroundingBlocks(Dimension2D pos)
        {
            return new Dimension2D[] {
                new Dimension2D(pos.X, pos.Y - 1), // north
                new Dimension2D(pos.X + 1, pos.Y), // east
                new Dimension2D(pos.X, pos.Y + 1), // south
                new Dimension2D(pos.X - 1, pos.Y)  // west
            };
        }
        public static Dimension2D ConvertToWorld(Vector2 pos)
        {
            // why
            // whyyy
            // WHYYYYYYYYYYYYYYYYYY DID I SWITCH X AND Y!
            return new Dimension2D((int)(pos.X / BLOCK_SIZE), (int)(pos.Y / BLOCK_SIZE));
        }

        public void Hurt(Dimension2D pos, float dmg)
        {
            blocks[pos.X, pos.Y].hurtTime = 0f;

            if (blocks[pos.X, pos.Y].canBeDestroyed)
            {
                blocks[pos.X, pos.Y].Reset();
            }
        }


        // inits & generation too
        public void LoadContent(ContentManager c)
        {
            ran = new Random();

            textures = new Texture2D[]
            {
                null,
                c.Load<Texture2D>("tex/crate"),
                c.Load<Texture2D>("tex/stone"),
                c.Load<Texture2D>("tex/item"),
                c.Load<Texture2D>("tex/bomb"),
                c.Load<Texture2D>("tex/fire")
            };

            procs = new Aiproc[4] { null, new Aiproc(), new Aiproc(), new Aiproc() };
            

            textManager = new TextManager(c.Load<SpriteFont>("font"));


            #region initial generation
            blocks = new Tile[SIZE, SIZE];

            for (byte x = 0; x < SIZE; x++)
                for (byte y = 0; y < SIZE; y++)
                {
                    blocks[x, y] = new Tile(0, new Vector2(x * BLOCK_SIZE, y * BLOCK_SIZE));
                }

            for (byte y = 0; y < SIZE; y++)
            {
                if (y == 0 || y == SIZE - 1)
                {
                    for (byte x = 0; x < SIZE; x++)
                    {
                        blocks[x, y].id = 2;
                    }
                }
                else
                {
                    blocks[0, y].id = 2;
                    blocks[SIZE - 1, y].id = 2;
                }
            }
            #endregion

            #region random generation(crates 'n stuff)

            for (byte x = 1; x < SIZE - 1; x++)
                for (byte y = 1; y < SIZE - 1; y++)
                {
                    if (ran.Next(0, 12) > 2)
                    {
                        blocks[x, y].id = 1;
                        blocks[x, y].canBeDestroyed = true;
                    }

                    if (ran.Next(0, 40) == 0)
                    {
                        blocks[x, y].id = 2;
                        blocks[x, y].canBeDestroyed = false;
                    }
                }

            #endregion

            #region gaps for players
            blocks[1, 1].id = 0;
            blocks[2, 1].id = 0;
            blocks[1, 2].id = 0;

            blocks[SIZE - 2, 1].id = 0;
            blocks[SIZE - 3, 1].id = 0;
            blocks[SIZE - 2, 2].id = 0;

            blocks[SIZE - 2, SIZE - 2].id = 0;
            blocks[SIZE - 3, SIZE - 2].id = 0;
            blocks[SIZE - 2, SIZE - 3].id = 0;

            blocks[1, SIZE - 2].id = 0;
            blocks[2, SIZE - 2].id = 0;
            blocks[1, SIZE - 3].id = 0;
            #endregion

            #region items
            for (byte i = 0; i < 4; i++)
            {
                Dimension2D pos = new Dimension2D();
                do
                {
                    pos.X = ran.Next(1, SIZE);
                    pos.Y = ran.Next(1, SIZE);
                }
                while (IsThereBlockAt(pos));

                CreateItem(pos);
            }
            #endregion
        }

        void CreateItem(Dimension2D pos)
        {
            Item i = new Item(new Vector2(pos.X * BLOCK_SIZE, pos.Y * BLOCK_SIZE));
            i.requiredTime = ran.Next(1000, 1600) * 10;
            i.effect = (ItemEffect)ran.Next(0, (int)ItemEffect.Last);
            i.color = new Color(ran.Next(256), ran.Next(256), ran.Next(256));

            blocks[pos.X, pos.Y] = i;
        }

        bool MakeExplosion(Vector2 pos, Dimension2D dim, ref Agent[] agents)
        {
            List<FireParticle> fire = new List<FireParticle>();

            // uhh... CTRL C? never heard of that key.
            for (byte i = 1; i < Bomb.Radius; i++)
            {
                Dimension2D d = new Dimension2D(dim.X + i, dim.Y);
                if (IsThereBlockAt(d))
                {
                    if (blocks[d.X, d.Y].canBeDestroyed)
                    {
                        blocks[d.X, d.Y].Reset();
                        fire.Add(new FireParticle(new Vector2(pos.X + i * BLOCK_SIZE, pos.Y)));
                    }
                    else
                    {
                        blocks[d.X, d.Y].hurtTime = 0f;
                    }
                    break;
                }
                fire.Add(new FireParticle(new Vector2(pos.X + i * BLOCK_SIZE, pos.Y)));
            }

            for (byte i = 1; i < Bomb.Radius; i++)
            {
                Dimension2D d = new Dimension2D(dim.X, dim.Y + i);
                if (IsThereBlockAt(d))
                {
                    if (blocks[d.X, d.Y].canBeDestroyed)
                    {
                        blocks[d.X, d.Y].Reset();
                        fire.Add(new FireParticle(new Vector2(pos.X, pos.Y + i * BLOCK_SIZE)));
                    }
                    else
                    {
                        blocks[d.X, d.Y].hurtTime = 0f;
                    }
                    break;
                }
                fire.Add(new FireParticle(new Vector2(pos.X, pos.Y + i * BLOCK_SIZE)));
            }

            for (byte i = 1; i < Bomb.Radius; i++)
            {
                Dimension2D d = new Dimension2D(dim.X - i, dim.Y);
                if (IsThereBlockAt(d))
                {
                    if (blocks[d.X, d.Y].canBeDestroyed)
                    {
                        blocks[d.X, d.Y].Reset();
                        fire.Add(new FireParticle(new Vector2(pos.X - i * BLOCK_SIZE, pos.Y)));
                    }
                    else
                    {
                        blocks[d.X, d.Y].hurtTime = 0f;
                    }
                    break;
                }
                fire.Add(new FireParticle(new Vector2(pos.X - i * BLOCK_SIZE, pos.Y)));
            }

            for (byte i = 1; i < Bomb.Radius; i++)
            {
                Dimension2D d = new Dimension2D(dim.X, dim.Y - i);
                if (IsThereBlockAt(d))
                {
                    if (blocks[d.X, d.Y].canBeDestroyed)
                    {
                        blocks[d.X, d.Y].Reset();
                        fire.Add(new FireParticle(new Vector2(pos.X, pos.Y - i * BLOCK_SIZE)));
                    }
                    else
                    {
                        blocks[d.X, d.Y].hurtTime = 0f;
                    }
                    break;
                }
                fire.Add(new FireParticle(new Vector2(pos.X, pos.Y - i * BLOCK_SIZE)));
            }
            
            // don't forget the fire on the start itself!
            fire.Add(new FireParticle(pos));

            bool ret = false;

            // collisions
            for (byte a = 0; a < agents.Length; a++)
            {
                if (agents[a] == null) continue;

                Vector2 ap = agents[a].GetPos();
                for (int i = 0; i < fire.Count; i++)
                {
                    Vector2 p = fire[i].GetPos();
                    if (ap.X + BLOCK_SIZE > p.X && ap.X < p.X + BLOCK_SIZE &&
                        ap.Y + BLOCK_SIZE > p.Y && ap.Y < p.Y + BLOCK_SIZE)
                    {
                        if (agents[a].Hurt(Bomb.Damage))
                        {
                            if (a == 0)
                            {
                                ret = true;
                                textManager.Set("You lost!");
                                agents[0] = null;
                            }
                            else
                            {
                                agents[a] = null;

                                bool won = agents[0] == null ? false : true;
                                if (won) for (byte b = 1; b < agents.Length; b++)
                                {
                                    if (agents[b] != null)
                                    {
                                        won = false;
                                        break;
                                    }
                                }

                                if (won)
                                {
                                    textManager.Set("You won!");
                                }
                                else
                                {
                                    textManager.Set("Player " + (a + 1).ToString() + " has been molten!");
                                }
                            }
                        }

                        break;
                    }
                }
            }

            fires.AddRange(fire);
            return ret;
        }

        public void DoAi(ref Agent[] agents, ref World world)
        {
            for (byte i = 1; i < agents.Length; i++)
            {
                if (agents[i] == null || agents[i].ProcessingMoves) continue;

                // check surrounding blocks
                Dimension2D pos = agents[i].GetWorldPos();

                if (procs[i].bombAction)
                {
                    if (procs[i].bombplaceTile != null)
                    {
                        agents[i].PlaceBomb(ref world, i);
                        procs[i].bombplaceTile = null;
                        agents[i].MoveTo(procs[i].startingTile);
                        
                    }
                    else if (procs[i].startingTile != null)
                    {
                        procs[i].startingTile = null;
                        agents[i].MoveTo(procs[i].coverTile);
                    }
                    else
                    {
                        agents[i].addWalkTime = 1600;//3000 - 700 * 2
                        procs[i].coverTile = null;
                        procs[i].bombAction = false;
                        procs[i].justBombed = true;
                        procs[i].prefer = 3;
                    }
                }
                else if (procs[i].justBombed)
                {
                    bool done = false;

                    for (byte z = 0; z < 2; z++)
                    {
                        Dimension2D target = new Dimension2D(pos.X + procs[i].dir.X, pos.Y + procs[i].dir.Y);

                        if (CanWalkHere(target))
                        {
                            if (procs[i].prefer > 0) procs[i].prefer--;
                            agents[i].MoveTo(target);

                            done = true;
                            break;
                        }
                        else
                        {
                            if (ran.Next(2) == 0)
                            {
                                procs[i].dir.X = ran.Next(2) == 0 ? -1 : 1;
                            }
                            else
                            {
                                procs[i].dir.Y = ran.Next(2) == 0 ? -1 : 1;
                            }
                        }
                    }

                    if (!done)
                    {
                        procs[i].prefer = 0;
                        procs[i].justBombed = false;
                        procs[i].dir = null;
                    }
                }
                //else
                if (!procs[i].bombAction)
                {
                    Dimension2D[] tiles = GetSurroundingBlocks(pos);

                    bool did = false;

                    if (procs[i].prefer == 0)
                    {
                        byte cnt = 0;

                        do
                        {
                            byte a = (byte)ran.Next(0, tiles.Length);

                            int next = a + 1 == tiles.Length ? 0 : a + 1;

                            // if there's a corner where you can cover yourself for a bomb
                            if (CanWalkHere(tiles[a]))
                            {
                                if (CanWalkHere(tiles[next]))
                                {
                                    if (blocks[tiles[next].X, tiles[next].Y].id == 3)
                                    {
                                        agents[i].MoveTo(tiles[next]);
                                    }
                                    else
                                    {
                                        procs[i].startingTile = pos;
                                        procs[i].coverTile = tiles[next];
                                        procs[i].bombplaceTile = tiles[a];
                                        procs[i].bombAction = true;
                                        agents[i].MoveTo(procs[i].bombplaceTile);
                                    }

                                    did = true;
                                    break;
                                }
                            }

                            cnt++;
                        }
                        while (cnt < 8);
                    }

                    // will say kaboom if you place 4 blocks arround it
                    if (!did)
                    {
                        Dimension2D p;

                        do
                        {
                            p = tiles[ran.Next(0, tiles.Length)];
                        }
                        while (!CanWalkHere(p));

                        agents[i].MoveTo(p);

                        Dimension2D d = new Dimension2D();
                        d.X = pos.X - p.X;
                        d.Y = pos.Y - p.Y;

                        procs[i].justBombed = true;
                        procs[i].dir = d;
                        procs[i].prefer = 0;
                    }
                    
                }
            }
        }

        public void Update(float delta, ref Agent[] agents, ref bool gameover)
        {
            for (byte x = 0; x < SIZE; x++)
                for (byte y = 0; y < SIZE; y++)
                {
                    switch (blocks[x, y].id)
                    {
                        case 0: break;
                        case 4:
                            Bomb b = (Bomb)blocks[x, y];
                            b.hurtTime += delta;

                            if (b.hurtTime >= 3000.0f)
                            {
                                // ignite
                                if (MakeExplosion(b.pos, new Dimension2D(x, y), ref agents))
                                {
                                    gameover = true;
                                }

                                Sound.PlayRandom("explosion");

                                byte n = b.GetOwnerIndex();
                                
                                if (agents[n] != null)
                                    agents[n].bombsPlaced--;
                                blocks[x, y].Reset();
                            }
                            break;
                        case 3:
                            Item i = (Item)blocks[x, y];
                            i.hurtTime += delta;

                            if (i.hurtTime >= i.requiredTime)
                            {
                                i.hurtTime = 0;

                                Dimension2D pos = GetSurroundingBlocks(new Dimension2D(x, y))[ran.Next(0, 4)];
                                
                                if (!IsThereBlockAt(pos))
                                {
                                    CreateItem(pos);
                                }
                            }
                            break;

                        default:
                            if (blocks[x, y].hurtTime < 255f)
                            {
                                blocks[x, y].hurtTime += delta;
                                
                                if (blocks[x, y].hurtTime >= 255f)
                                {
                                    blocks[x, y].color = Color.White;
                                }
                                else
                                {
                                    blocks[x, y].color.R = (byte)blocks[x, y].hurtTime;
                                }
                            }
                            break;
                    }
                }

            // collision
            for (byte i = 0; i < agents.Length; i++)
            {
                if (agents[i] == null) continue;
                Dimension2D pos = agents[i].GetWorldPos();

                Tile t = blocks[pos.X, pos.Y];

                if (t.id == 3)
                {
                    Item e = (Item)t;

                    switch (e.effect)
                    {
                        default:
                            break;
                        case ItemEffect.IncreaseExplosion:
                            Bomb.IncreaseRadius();
                            textManager.Set("Explosion radius increased!");
                            break;
                        case ItemEffect.IncreaseDamage:
                            Bomb.IncreaseDamage();
                            textManager.Set("Explosion damage increased!");
                            break;
                        case ItemEffect.Speed:
                            if (agents[i].speed <= 450)
                            {
                                agents[i].speed += 50;
                                if (i == 0) textManager.Set("You now have more speed!");
                            }
                            else
                            {
                                if (i == 0) textManager.Set("Can't go faster!");
                            }
                            break;
                        case ItemEffect.ExtraBomb:
                            agents[i].maxBombs++;
                            if (i == 0) textManager.Set("You have an extra bomb!");
                            break;
                        case ItemEffect.Heal:
                            agents[i].health += 25.0f;
                            if (i == 0) textManager.Set("You just got healed!");
                            break;
                    }

                    Sound.PlayRandom("powerup");
                    blocks[pos.X, pos.Y].Reset();
                }
            }

            // fires
            for (int i = 0; i < fires.Count; i++)
            {
                if (fires[i].Update(delta))
                {
                    fires.RemoveAt(i);
                    i--;
                    continue;
                }
            }
            
            // text
            textManager.Update(delta);
        }

        public void Draw(SpriteBatch sb)
        {
            // tiles
            for (byte x = 0; x < SIZE; x++)
                for (byte y = 0; y < SIZE; y++)
                {
                    if (blocks[x, y].id != 0)
                    {
                        sb.Draw(textures[blocks[x, y].id], blocks[x, y].pos, blocks[x, y].color);
                    }
                }

            // fires
            for (int i = 0; i < fires.Count; i++)
            {
                fires[i].Draw(sb, textures[5]);
            }

            // text
            textManager.Draw(sb);
        }
    }
}
